@@ -13,7 +13,7 @@
 
 int checkCert(GWEN_GUI *gui,
 	      const GWEN_SSLCERTDESCR *cert,
-	      GWEN_IO_LAYER *io,
+	      GWEN_SYNCIO *sio,
 	      uint32_t guiid) {
   /* automatically accept any cert for non-interactive checks */
   return 0;
@@ -29,32 +29,17 @@ int main(int argc, char **argv) {
   GWEN_Gui_SetCheckCertFn(gui, checkCert);
   GWEN_Gui_SetGui(gui);
 
-  return check_io_tls();
-
-
   if (check_db())
     errs++;
 
-  if (check_io_file())
+  if (check_syncio_tls())
     errs++;
 
-  if (check_io_buffered())
+  if (check_syncio_http())
+    errs++;
+  if (check_syncio_https())
     errs++;
 
-  if (check_io_socket())
-    errs++;
-
-  if (check_io_tls())
-    errs++;
-
-  if (check_io_http())
-    errs++;
-
-  if (check_io_https())
-    errs++;
-
-  if (check_io_packets())
-    errs++;
 
   if (errs)
     return 2;
