@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Fri Sep 12 2003
-    copyright   : (C) 2003 by Martin Preuss
+    copyright   : (C) 2003-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -47,16 +44,14 @@ GWEN_BUFFER *GWEN_Buffer_new(char *buffer,
   GWEN_NEW_OBJECT(GWEN_BUFFER, bf);
   if (!buffer) {
     /* allocate buffer */
-    if (size) {
-      bf->realPtr=(char*)GWEN_Memory_malloc(size+1);
-      assert(bf->realPtr);
-      bf->ptr=bf->realPtr;
-      bf->realBufferSize=size+1;
-      bf->bufferSize=size+1;
-      bf->flags=GWEN_BUFFER_FLAGS_OWNED;
-      bf->bytesUsed=used;
-      bf->ptr[0]=0;
-    }
+    bf->realPtr=(char*)GWEN_Memory_malloc(size+1);
+    assert(bf->realPtr);
+    bf->ptr=bf->realPtr;
+    bf->realBufferSize=size+1;
+    bf->bufferSize=size+1;
+    bf->flags=GWEN_BUFFER_FLAGS_OWNED;
+    bf->bytesUsed=used;
+    bf->ptr[0]=0;
   }
   else {
     /* use existing buffer */
@@ -81,10 +76,9 @@ void GWEN_Buffer_free(GWEN_BUFFER *bf){
   if (bf) {
     if (bf->flags & GWEN_BUFFER_FLAGS_OWNED)
       GWEN_Memory_dealloc(bf->realPtr);
-    if (bf->bio) {
-      if (bf->flags & GWEN_BUFFER_FLAGS_OWN_BIO) {
-        GWEN_BufferedIO_free(bf->bio);
-      }
+    if (bf->syncIo) {
+      if (bf->flags & GWEN_BUFFER_FLAGS_OWN_SYNCIO)
+        GWEN_SyncIo_free(bf->syncIo);
     }
     GWEN_FREE_OBJECT(bf);
   }
@@ -238,15 +232,14 @@ int GWEN_Buffer_SetPos(GWEN_BUFFER *bf, uint32_t i){
   assert(bf);
 
   if (i>=bf->bufferSize) {
-    if ((bf->mode & GWEN_BUFFER_MODE_USE_BIO) ||
-	(bf->mode & GWEN_BUFFER_MODE_USE_IO)) {
+    if (bf->mode & GWEN_BUFFER_MODE_USE_SYNCIO) {
       bf->pos=i;
     }
     else {
       DBG_ERROR(GWEN_LOGDOMAIN,
                 "Position %d outside buffer boundaries (%d bytes)",
                 i, bf->bufferSize);
-      return -1;
+      return GWEN_ERROR_BUFFER_OVERFLOW;
     }
   }
   bf->pos=i;
@@ -262,25 +255,15 @@ uint32_t GWEN_Buffer_GetUsedBytes(GWEN_BUFFER *bf){
 
 
 
-int GWEN_Buffer_SetUsedBytes(GWEN_BUFFER *bf, uint32_t i){
-  assert(bf);
-
-  DBG_WARN(GWEN_LOGDOMAIN,
-           "GWEN_Buffer_SetUsedBytes: Deprecated, "
-           "please use GWEN_Buffer_Crop instead.");
-  if (i>bf->bufferSize) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Bytes used>buffer size (%d>%d bytes)",
-              i, bf->bufferSize);
-    return 1;
-  }
-  bf->bytesUsed=i;
-  return 0;
-}
-
-
-
 int GWEN_Buffer_AllocRoom(GWEN_BUFFER *bf, uint32_t size) {
   assert(bf);
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
   /*DBG_VERBOUS(GWEN_LOGDOMAIN, "Allocating %d bytes", size);*/
   /*if (bf->pos+size>bf->bufferSize) {*/
   if (bf->bytesUsed+(size+1) > bf->bufferSize) {
@@ -296,7 +279,7 @@ int GWEN_Buffer_AllocRoom(GWEN_BUFFER *bf, uint32_t size) {
       if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
         abort();
       }
-      return 1;
+      return GWEN_ERROR_BUFFER_OVERFLOW;
     }
 
     /* calculate reserved bytes (to set ptr later) */
@@ -318,7 +301,7 @@ int GWEN_Buffer_AllocRoom(GWEN_BUFFER *bf, uint32_t size) {
       if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
         abort();
       }
-      return 1;
+      return GWEN_ERROR_BUFFER_OVERFLOW;
     }
     DBG_VERBOUS(GWEN_LOGDOMAIN, "Reallocating from %d to %d bytes",
                 bf->bufferSize, nsize);
@@ -334,7 +317,7 @@ int GWEN_Buffer_AllocRoom(GWEN_BUFFER *bf, uint32_t size) {
       if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
 	abort();
       }
-      return 1;
+      return GWEN_ERROR_MEMORY_FULL;
     }
 
     /* store new size and pointer */
@@ -352,10 +335,22 @@ int GWEN_Buffer_AllocRoom(GWEN_BUFFER *bf, uint32_t size) {
 int GWEN_Buffer_AppendBytes(GWEN_BUFFER *bf,
                             const char *buffer,
                             uint32_t size){
+  int rv;
+
   assert(bf);
-  if (GWEN_Buffer_AllocRoom(bf, size+1)) {
+
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  rv=GWEN_Buffer_AllocRoom(bf, size+1);
+  if (rv<0) {
     DBG_DEBUG(GWEN_LOGDOMAIN, "called from here");
-    return 1;
+    return rv;
   }
   /* if (bf->pos+size>bf->bufferSize) { */
   if (bf->bytesUsed+size>bf->bufferSize) {
@@ -363,9 +358,9 @@ int GWEN_Buffer_AppendBytes(GWEN_BUFFER *bf,
               /*bf->pos, size,*/
               bf->bytesUsed, size+1,
               bf->bufferSize);
-    return 1;
+    return GWEN_ERROR_BUFFER_OVERFLOW;
   }
-  /*memmove(bf->ptr+bf->pos, buffer, size);*/
+
   memmove(bf->ptr+bf->bytesUsed, buffer, size);
   /*bf->pos+=size;*/
   if (bf->pos==bf->bytesUsed)
@@ -379,17 +374,28 @@ int GWEN_Buffer_AppendBytes(GWEN_BUFFER *bf,
 
 
 int GWEN_Buffer_AppendByte(GWEN_BUFFER *bf, char c){
+  int rv;
+
   assert(bf);
 
-  if (GWEN_UNLIKELY(bf->bytesUsed+1+1 > bf->bufferSize)) {
-    if (GWEN_UNLIKELY(GWEN_Buffer_AllocRoom(bf, 1+1))) {
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  if (bf->bytesUsed+1+1 > bf->bufferSize) {
+    rv=GWEN_Buffer_AllocRoom(bf, 1+1);
+    if (rv<0) {
       DBG_DEBUG(GWEN_LOGDOMAIN, "here");
-      return 1;
+      return rv;
     }
   }
 
   bf->ptr[bf->bytesUsed]=c;
-  if (GWEN_LIKELY(bf->pos == bf->bytesUsed))
+  if (bf->pos == bf->bytesUsed)
     bf->pos++;
   /* append a NULL to allow using the buffer as ASCIIZ string */
   bf->ptr[++(bf->bytesUsed)]=0;
@@ -398,32 +404,33 @@ int GWEN_Buffer_AppendByte(GWEN_BUFFER *bf, char c){
 
 
 
-int GWEN_Buffer__FillBuffer_Bio(GWEN_BUFFER *bf){
-  if (bf->bio) {
-    unsigned int toread;
-    int gerr;
+int GWEN_Buffer__FillBuffer_SyncIo(GWEN_BUFFER *bf){
+  if (bf->syncIo) {
+    uint32_t toread;
+    int rv;
 
-    if (GWEN_BufferedIO_CheckEOF(bf->bio)) {
-      DBG_INFO(GWEN_LOGDOMAIN, "End of data stream reached");
-      return GWEN_ERROR_EOF;
-    }
     toread=bf->pos-bf->bytesUsed+1;
     if (GWEN_Buffer_AllocRoom(bf, toread+1)) {
       DBG_INFO(GWEN_LOGDOMAIN, "Buffer too small");
-      return -1;
+      return GWEN_ERROR_GENERIC;
     }
-    gerr=GWEN_BufferedIO_ReadRawForced(bf->bio,
-				       bf->ptr+bf->bytesUsed,
-				       &toread);
-    if (gerr) {
-      DBG_INFO_ERR(GWEN_LOGDOMAIN, gerr);
-      return -1;
+    rv=GWEN_SyncIo_ReadForced(bf->syncIo,
+			      (uint8_t*) (bf->ptr+bf->bytesUsed),
+			      toread);
+    if (rv<0) {
+      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      return rv;
     }
-    bf->bytesUsed+=toread;
+    else if (rv==0) {
+      DBG_INFO(GWEN_LOGDOMAIN, "EOF met");
+      return GWEN_ERROR_EOF;
+    }
+
+    bf->bytesUsed+=rv;
   }
   else {
     DBG_DEBUG(GWEN_LOGDOMAIN,
-	      "End of used area reached and no BIO (%d bytes)",
+	      "End of used area reached and no SYNCIO (%d bytes)",
 	      bf->pos);
     return GWEN_ERROR_EOF;
   }
@@ -432,53 +439,10 @@ int GWEN_Buffer__FillBuffer_Bio(GWEN_BUFFER *bf){
 
 
 
-int GWEN_Buffer__FillBuffer_IoLayer(GWEN_BUFFER *bf){
-  if (bf->ioLayer) {
-    unsigned int toread;
-    int rv;
-
-    toread=bf->pos-bf->bytesUsed+1;
-    if (GWEN_Buffer_AllocRoom(bf, toread+1)) {
-      DBG_INFO(GWEN_LOGDOMAIN, "Buffer too small");
-      return -1;
-    }
-    /* wait up to 30 secs for available data */
-    rv=GWEN_Io_Layer_ReadBytes(bf->ioLayer,
-			       (uint8_t*)bf->ptr+bf->bytesUsed,
-			       toread,
-			       GWEN_IO_REQUEST_FLAGS_READALL,
-			       0, 30000);
-    if (rv<(int)toread) {
-      if (rv<0) {
-	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-        return rv;
-      }
-      else {
-	DBG_INFO(GWEN_LOGDOMAIN, "EOF met");
-	/* still some bytes available, but not all */
-	bf->bytesUsed+=toread;
-	return GWEN_ERROR_EOF;
-      }
-    }
-    bf->bytesUsed+=toread;
-    return 0;
-  }
-  else {
-    DBG_DEBUG(GWEN_LOGDOMAIN,
-	      "End of used area reached and no BIO (%d bytes)",
-	      bf->pos);
-    return GWEN_ERROR_EOF;
-  }
-}
-
-
-
 int GWEN_Buffer__FillBuffer(GWEN_BUFFER *bf){
   assert(bf);
-  if (bf->mode & GWEN_BUFFER_MODE_USE_BIO)
-    return GWEN_Buffer__FillBuffer_Bio(bf);
-  else if (bf->mode & GWEN_BUFFER_MODE_USE_IO)
-    return GWEN_Buffer__FillBuffer_IoLayer(bf);
+  if (bf->mode & GWEN_BUFFER_MODE_USE_SYNCIO)
+    return GWEN_Buffer__FillBuffer_SyncIo(bf);
   else {
     DBG_DEBUG(GWEN_LOGDOMAIN,
 	      "End of used area reached (%d bytes)", bf->pos);
@@ -492,8 +456,13 @@ int GWEN_Buffer_PeekByte(GWEN_BUFFER *bf){
   assert(bf);
 
   if (bf->pos>=bf->bytesUsed) {
-    if (GWEN_Buffer__FillBuffer(bf))
-      return -1;
+    int rv;
+
+    rv=GWEN_Buffer__FillBuffer(bf);
+    if (rv<0) {
+      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      return rv;
+    }
   }
 
   return (unsigned char) (bf->ptr[bf->pos]);
@@ -505,8 +474,13 @@ int GWEN_Buffer_ReadByte(GWEN_BUFFER *bf){
   assert(bf);
 
   if (bf->pos>=bf->bytesUsed) {
-    if (GWEN_Buffer__FillBuffer(bf))
-      return -1;
+    int rv;
+
+    rv=GWEN_Buffer__FillBuffer(bf);
+    if (rv<0) {
+      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      return rv;
+    }
   }
 
   return (unsigned char) (bf->ptr[bf->pos++]);
@@ -518,7 +492,7 @@ int GWEN_Buffer_IncrementPos(GWEN_BUFFER *bf, uint32_t i){
   assert(bf);
 
   if (i+bf->pos>=bf->bufferSize) {
-    if (!(bf->mode & GWEN_BUFFER_MODE_USE_BIO)) {
+    if (!(bf->mode & GWEN_BUFFER_MODE_USE_SYNCIO)) {
       DBG_DEBUG(GWEN_LOGDOMAIN,
                 "Position %d outside buffer boundaries (%d bytes)\n"
                 "Incrementing anyway",
@@ -547,7 +521,7 @@ int GWEN_Buffer_AdjustUsedBytes(GWEN_BUFFER *bf){
   else {
     DBG_ERROR(GWEN_LOGDOMAIN, "Pointer outside buffer size (%d bytes)",
               bf->bufferSize);
-    return 1;
+    return GWEN_ERROR_BUFFER_OVERFLOW;
   }
 }
 
@@ -560,7 +534,7 @@ int GWEN_Buffer_DecrementPos(GWEN_BUFFER *bf, uint32_t i){
     DBG_ERROR(GWEN_LOGDOMAIN,
               "Position %d outside buffer boundaries (%d bytes)",
               bf->pos-i, bf->bufferSize);
-    return 1;
+    return GWEN_ERROR_BUFFER_OVERFLOW;
   }
   bf->pos-=i;
   return 0;
@@ -568,10 +542,19 @@ int GWEN_Buffer_DecrementPos(GWEN_BUFFER *bf, uint32_t i){
 
 
 
-int GWEN_Buffer_AppendBuffer(GWEN_BUFFER *bf,
-                             GWEN_BUFFER *sf){
+int GWEN_Buffer_AppendBuffer(GWEN_BUFFER *bf, GWEN_BUFFER *sf){
+
   assert(bf);
   assert(sf);
+
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
   if (sf->bytesUsed)
     return GWEN_Buffer_AppendBytes(bf, sf->ptr, sf->bytesUsed);
   return 0;
@@ -622,62 +605,64 @@ void GWEN_Buffer_SetBookmark(GWEN_BUFFER *bf, unsigned int idx,
 
 
 
-void GWEN_Buffer_Dump(GWEN_BUFFER *bf, FILE *f, unsigned int insert) {
+void GWEN_Buffer_Dump(GWEN_BUFFER *bf, unsigned int insert) {
   uint32_t k;
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Buffer:\n");
+    fprintf(stderr, " ");
+  fprintf(stderr, "Buffer:\n");
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Pos            : %d (%04x)\n", bf->pos, bf->pos);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Pos            : %d (%04x)\n", bf->pos, bf->pos);
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Buffer Size    : %d\n", bf->bufferSize);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Buffer Size    : %d\n", bf->bufferSize);
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Hard limit     : %d\n", bf->hardLimit);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Hard limit     : %d\n", bf->hardLimit);
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Bytes Used     : %d\n", bf->bytesUsed);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Bytes Used     : %d\n", bf->bytesUsed);
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Bytes Reserved : %u\n",
+    fprintf(stderr, " ");
+  fprintf(stderr, "Bytes Reserved : %u\n",
           (uint32_t)(bf->ptr-bf->realPtr));
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Flags          : %08x ( ", bf->flags);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Flags          : %08x ( ", bf->flags);
   if (bf->flags & GWEN_BUFFER_FLAGS_OWNED)
-    fprintf(f, "OWNED ");
-  fprintf(f, ")\n");
+    fprintf(stderr, "OWNED ");
+  fprintf(stderr, ")\n");
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Mode           : %08x ( ", bf->mode);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Mode           : %08x ( ", bf->mode);
   if (bf->mode & GWEN_BUFFER_MODE_DYNAMIC)
-    fprintf(f, "DYNAMIC ");
+    fprintf(stderr, "DYNAMIC ");
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY)
+    fprintf(stderr, "READONLY ");
   if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL)
-    fprintf(f, "ABORT_ON_MEMFULL ");
-  fprintf(f, ")\n");
+    fprintf(stderr, "ABORT_ON_MEMFULL ");
+  fprintf(stderr, ")\n");
 
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Bookmarks      :");
+    fprintf(stderr, " ");
+  fprintf(stderr, "Bookmarks      :");
   for (k=0; k<GWEN_BUFFER_MAX_BOOKMARKS; k++)
-    fprintf(f, " %d", bf->bookmarks[k]);
-  fprintf(f, "\n");
+    fprintf(stderr, " %d", bf->bookmarks[k]);
+  fprintf(stderr, "\n");
 
   if (bf->ptr && bf->bytesUsed) {
     for (k=0; k<insert; k++)
-      fprintf(f, " ");
-    fprintf(f, "Data:\n");
-    GWEN_Text_DumpString(bf->ptr, bf->bytesUsed, f, insert+1);
+      fprintf(stderr, " ");
+    fprintf(stderr, "Data:\n");
+    GWEN_Text_DumpString(bf->ptr, bf->bytesUsed, insert+1);
   }
 }
 
@@ -699,28 +684,7 @@ void GWEN_Buffer_Rewind(GWEN_BUFFER *bf){
 
 
 
-int GWEN_Buffer_ReadBytes(GWEN_BUFFER *bf,
-                          char *buffer,
-                          uint32_t *size){
-#if 0
-  /* old behaviour */
-  uint32_t i;
-  int c;
-
-  i=0;
-
-  while(i<*size) {
-    c=GWEN_Buffer_ReadByte(bf);
-    if (c==-1)
-      break;
-    buffer[i]=c;
-    i++;
-  } /* while */
-
-  *size=i;
-  return 0;
-
-#else
+int GWEN_Buffer_ReadBytes(GWEN_BUFFER *bf, char *buffer, uint32_t *size){
   /* optimized for speed */
   uint32_t i;
   char *pdst;
@@ -756,7 +720,6 @@ int GWEN_Buffer_ReadBytes(GWEN_BUFFER *bf,
   *size=i;
   DBG_VERBOUS(GWEN_LOGDOMAIN, "Copied %d bytes", *size);
   return 0;
-#endif
 }
 
 
@@ -793,13 +756,20 @@ int GWEN_Buffer_InsertRoom(GWEN_BUFFER *bf,
                            uint32_t size){
   char *p;
   int i;
+  int rv;
 
   assert(bf);
 
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
   if (bf->pos==0) {
     if (bf->bytesUsed==0) {
-      int rv;
-
       /* no bytes used, simply return */
       rv=GWEN_Buffer_AllocRoom(bf, size);
       if (rv) {
@@ -823,15 +793,16 @@ int GWEN_Buffer_InsertRoom(GWEN_BUFFER *bf,
     }
   }
 
-  if (GWEN_Buffer_AllocRoom(bf, size)) {
+  rv=GWEN_Buffer_AllocRoom(bf, size);
+  if (rv<0) {
     DBG_DEBUG(GWEN_LOGDOMAIN, "called from here");
-    return 1;
+    return rv;
   }
   if (bf->pos+size>bf->bufferSize) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Buffer full (%d [%d] of %d bytes)",
 	      bf->pos, size,
 	      bf->bufferSize);
-    return -1;
+    return GWEN_ERROR_BUFFER_OVERFLOW;
   }
   p=bf->ptr+bf->pos;
   i=bf->bytesUsed-bf->pos;
@@ -853,6 +824,14 @@ int GWEN_Buffer_RemoveRoom(GWEN_BUFFER *bf, uint32_t size){
 
   assert(bf);
 
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
   if (bf->pos==0) {
     if (bf->bytesUsed<size) {
       /* can't remove bytes we don't have */
@@ -892,6 +871,14 @@ int GWEN_Buffer_ReplaceBytes(GWEN_BUFFER *bf,
   int32_t d;
   int rv;
 
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
   /* either insert or remove bytes */
   d=size-rsize;
   if (d<0) {
@@ -921,11 +908,23 @@ int GWEN_Buffer_ReplaceBytes(GWEN_BUFFER *bf,
 int GWEN_Buffer_InsertBytes(GWEN_BUFFER *bf,
                             const char *buffer,
                             uint32_t size){
+  int rv;
+
   assert(bf);
   assert(buffer);
 
-  if (GWEN_Buffer_InsertRoom(bf, size)) {
-    return -1;
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  rv=GWEN_Buffer_InsertRoom(bf, size);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    return rv;
   }
   memmove(bf->ptr+bf->pos, buffer, size);
   return 0;
@@ -934,10 +933,22 @@ int GWEN_Buffer_InsertBytes(GWEN_BUFFER *bf,
 
 
 int GWEN_Buffer_InsertByte(GWEN_BUFFER *bf, char c){
+  int rv;
+
   assert(bf);
 
-  if (GWEN_Buffer_InsertRoom(bf, 1)) {
-    return -1;
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  rv=GWEN_Buffer_InsertRoom(bf, 1);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    return rv;
   }
   bf->ptr[bf->pos]=c;
   return 0;
@@ -958,6 +969,15 @@ int GWEN_Buffer_InsertBuffer(GWEN_BUFFER *bf,
 int GWEN_Buffer_Crop(GWEN_BUFFER *bf,
                      uint32_t pos,
                      uint32_t l) {
+
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
   if (pos>=bf->bufferSize) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Position outside buffer");
     return -1;
@@ -1000,38 +1020,20 @@ int GWEN_Buffer_InsertString(GWEN_BUFFER *bf,
 
 
 
-void GWEN_Buffer_SetSourceBIO(GWEN_BUFFER *bf,
-                              GWEN_BUFFEREDIO *bio,
-                              int take){
+void GWEN_Buffer_SetSourceSyncIo(GWEN_BUFFER *bf,
+				 GWEN_SYNCIO *sio,
+				 int take) {
   assert(bf);
-  if (bf->bio) {
-    if (bf->flags & GWEN_BUFFER_FLAGS_OWN_BIO) {
-      GWEN_BufferedIO_free(bf->bio);
+  if (bf->syncIo) {
+    if (bf->flags & GWEN_BUFFER_FLAGS_OWN_SYNCIO) {
+      GWEN_SyncIo_free(bf->syncIo);
     }
   }
   if (take)
-    bf->flags|=GWEN_BUFFER_FLAGS_OWN_BIO;
+    bf->flags|=GWEN_BUFFER_FLAGS_OWN_SYNCIO;
   else
-    bf->flags&=~GWEN_BUFFER_FLAGS_OWN_BIO;
-  bf->bio=bio;
-}
-
-
-
-void GWEN_Buffer_SetSourceIoLayer(GWEN_BUFFER *bf,
-				  GWEN_IO_LAYER *io,
-				  int take) {
-  assert(bf);
-  if (bf->ioLayer) {
-    if (bf->flags & GWEN_BUFFER_FLAGS_OWN_IO) {
-      GWEN_Io_Layer_free(bf->ioLayer);
-    }
-  }
-  if (take)
-    bf->flags|=GWEN_BUFFER_FLAGS_OWN_IO;
-  else
-    bf->flags&=~GWEN_BUFFER_FLAGS_OWN_IO;
-  bf->ioLayer=io;
+    bf->flags&=~GWEN_BUFFER_FLAGS_OWN_SYNCIO;
+  bf->syncIo=sio;
 }
 
 
@@ -1039,17 +1041,29 @@ void GWEN_Buffer_SetSourceIoLayer(GWEN_BUFFER *bf,
 int GWEN_Buffer_FillWithBytes(GWEN_BUFFER *bf,
                               unsigned char c,
                               uint32_t size){
+  int rv;
+
   assert(bf);
-  if (GWEN_Buffer_AllocRoom(bf, size+1)) {
+
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  rv=GWEN_Buffer_AllocRoom(bf, size+1);
+  if (rv<0) {
     DBG_DEBUG(GWEN_LOGDOMAIN, "called from here");
-    return 1;
+    return rv;
   }
   /* if (bf->pos+size>bf->bufferSize) { */
   if (bf->bytesUsed+size>bf->bufferSize) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Buffer full (%d [%d] of %d bytes)",
               bf->bytesUsed, size+1,
               bf->bufferSize);
-    return 1;
+    return GWEN_ERROR_BUFFER_OVERFLOW;
   }
   memset(bf->ptr+bf->bytesUsed, c, size);
   if (bf->pos==bf->bytesUsed)
@@ -1065,9 +1079,21 @@ int GWEN_Buffer_FillWithBytes(GWEN_BUFFER *bf,
 int GWEN_Buffer_FillLeftWithBytes(GWEN_BUFFER *bf,
                                   unsigned char c,
                                   uint32_t size){
+  int rv;
+
   assert(bf);
 
-  if (GWEN_Buffer_InsertRoom(bf, size)) {
+  if (bf->mode & GWEN_BUFFER_MODE_READONLY) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Read-only mode");
+    if (bf->mode & GWEN_BUFFER_MODE_ABORT_ON_MEMFULL) {
+      abort();
+      return GWEN_ERROR_PERMISSIONS;
+    }
+  }
+
+  rv=GWEN_Buffer_InsertRoom(bf, size);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
     return -1;
   }
   memset(bf->ptr+bf->pos, c, size);
