@@ -44,8 +44,7 @@ typedef struct GWEN_BUFFER GWEN_BUFFER;
 #endif
 
 #include <gwenhywfar/types.h>
-#include <gwenhywfar/bufferedio.h>
-#include <gwenhywfar/iolayer.h>
+#include <gwenhywfar/syncio.h>
 
 
 #include <stdio.h>
@@ -65,12 +64,12 @@ extern "C" {
  */
 /*@{*/
 
-#define GWEN_BUFFER_MAX_BOOKMARKS 32
+#define GWEN_BUFFER_MAX_BOOKMARKS 64
 
 #define GWEN_BUFFER_MODE_DYNAMIC          0x0001
 #define GWEN_BUFFER_MODE_ABORT_ON_MEMFULL 0x0002
-#define GWEN_BUFFER_MODE_USE_BIO          0x0004
-#define GWEN_BUFFER_MODE_USE_IO           0x0008
+#define GWEN_BUFFER_MODE_USE_SYNCIO       0x0010
+#define GWEN_BUFFER_MODE_READONLY         0x0020
 
 #define GWEN_BUFFER_MODE_DEFAULT \
   (\
@@ -258,12 +257,6 @@ void GWEN_Buffer_SetBookmark(GWEN_BUFFER *bf, unsigned int idx,
                              uint32_t v);
 
 
-/**
- * @return 0 if ok, !=0 on error
- */
-GWENHYWFAR_API
-int GWEN_Buffer_SetUsedBytes(GWEN_BUFFER *bf, uint32_t i);
-
 /**
  * Copies the contents of the given buffer to this GWEN_BUFFER, if there is
  * enough room.
@@ -523,33 +516,21 @@ int GWEN_Buffer_Crop(GWEN_BUFFER *bf,
 
 
 /**
- * Sets the buffered input to be used as a source.
- * This BIO is used when a byte is to be returned while the buffer is
- * empty (or the end of the buffer is reached). In such a case the missing
- * bytes are read from this BIO if the mode contains
- * @ref GWEN_BUFFER_MODE_USE_BIO.
- */
-GWENHYWFAR_API
-void GWEN_Buffer_SetSourceBIO(GWEN_BUFFER *bf,
-			      GWEN_BUFFEREDIO *bio,
-			      int take);
-
-/**
- * Sets the io layer to be used as a source.
+ * Sets the syncio to be used as a source.
  * This io layer is used when a byte is to be returned while the buffer is
  * empty (or the end of the buffer is reached). In such a case the missing
  * bytes are read from this io layer if the mode contains
- * @ref GWEN_BUFFER_MODE_USE_IO.
+ * @ref GWEN_BUFFER_MODE_USE_SYNCIO.
  */
 GWENHYWFAR_API
-void GWEN_Buffer_SetSourceIoLayer(GWEN_BUFFER *bf,
-				  GWEN_IO_LAYER *io,
-				  int take);
+void GWEN_Buffer_SetSourceSyncIo(GWEN_BUFFER *bf,
+				 GWEN_SYNCIO *sio,
+				 int take);
 
 
 /** Print the current content of buffer @c bf into the file @c f. */
 GWENHYWFAR_API
-void GWEN_Buffer_Dump(GWEN_BUFFER *bf, FILE *f, unsigned int insert);
+void GWEN_Buffer_Dump(GWEN_BUFFER *bf, unsigned int insert);
 
 
 /*@}*/
