@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Sat Nov 15 2003
     copyright   : (C) 2003 by Martin Preuss
     email       : martin@libchipcard.de
@@ -30,6 +27,9 @@
 # include <config.h>
 #endif
 
+#define DISABLE_DEBUGLOG
+
+
 #include "list_p.h"
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/debug.h>
@@ -181,6 +181,7 @@ GWEN_LIST *GWEN_List_dup(const GWEN_LIST *l){
   assert(l);
   assert(l->listPtr);
   GWEN_NEW_OBJECT(GWEN_LIST, nl);
+  GWEN_INHERIT_INIT(GWEN_LIST, nl);
   nl->listPtr=l->listPtr;
   GWEN__ListPtr_Attach(nl->listPtr);
   return nl;
@@ -404,7 +405,7 @@ void GWEN_List_Clear(GWEN_LIST *l){
     GWEN__LISTPTR *nlp;
 
     /* only copy the list if someone else is using it */
-    nlp=GWEN__ListPtr_new(l->listPtr);
+    nlp=GWEN__ListPtr_dup(l->listPtr);
     GWEN__ListPtr_free(l->listPtr);
     l->listPtr=nlp;
   }
@@ -551,6 +552,8 @@ GWEN_LIST_ITERATOR *GWEN_List_FindIter(GWEN_LIST *l, const void *p) {
   return 0;
 }
 
+
+
 const void *GWEN_List_Contains(GWEN_LIST *l, const void *p) {
   GWEN_LIST_ITERATOR *li;
 
@@ -562,12 +565,15 @@ const void *GWEN_List_Contains(GWEN_LIST *l, const void *p) {
   return 0;
 }
 
+
+
 void GWEN_List_Remove(GWEN_LIST *l, const void *p) {
   GWEN_LIST_ITERATOR *li;
 
   li = GWEN_List_FindIter(l, p);
   if (li) {
     GWEN_List_Erase(l, li);
+    GWEN_ListIterator_free(li);
   }
 }
 
