@@ -325,7 +325,6 @@ int GWEN_PluginManager_AddPathFromWinReg(GWEN_UNUSED GWEN_PLUGIN_MANAGER *pm,
 
 GWEN_PLUGIN *GWEN_PluginManager_LoadPlugin(GWEN_PLUGIN_MANAGER *pm,
                                            const char *modname){
-#ifdef ENABLE_PLUGIN_LOADING
   GWEN_LIBLOADER *ll;
   GWEN_PLUGIN *plugin;
   GWEN_PLUGIN_FACTORYFN fn;
@@ -407,10 +406,6 @@ GWEN_PLUGIN *GWEN_PluginManager_LoadPlugin(GWEN_PLUGIN_MANAGER *pm,
   GWEN_StringList_free(sl);
   GWEN_Plugin_SetLibLoader(plugin, ll);
   return plugin;
-#else
-  DBG_ERROR(GWEN_LOGDOMAIN, "Loading of plugins is disabled");
-  return NULL;
-#endif
 }
 
 
@@ -418,7 +413,6 @@ GWEN_PLUGIN *GWEN_PluginManager_LoadPlugin(GWEN_PLUGIN_MANAGER *pm,
 GWEN_PLUGIN *GWEN_PluginManager_LoadPluginFile(GWEN_PLUGIN_MANAGER *pm,
                                                const char *modname,
                                                const char *fname){
-#ifdef ENABLE_PLUGIN_LOADING
   GWEN_LIBLOADER *ll;
   GWEN_PLUGIN *plugin;
   GWEN_PLUGIN_FACTORYFN fn;
@@ -469,10 +463,6 @@ GWEN_PLUGIN *GWEN_PluginManager_LoadPluginFile(GWEN_PLUGIN_MANAGER *pm,
   GWEN_Plugin_SetLibLoader(plugin, ll);
 
   return plugin;
-#else
-  DBG_ERROR(GWEN_LOGDOMAIN, "Loading of plugins is disabled");
-  return NULL;
-#endif
 }
 
 
