@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
- -------------------
- cvs         : $Id$
  begin       : Thu Apr 03 2003
- copyright   : (C) 2003 by Martin Preuss
+ copyright   : (C) 2003-2010 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -29,12 +26,14 @@
 # include <config.h>
 #endif
 
+#define DISABLE_DEBUGLOG
+
+
 #include "plugindescr_p.h"
 #include "i18n_l.h"
 #include <gwenhywfar/buffer.h>
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/directory.h>
-#include <gwenhywfar/bio_buffer.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -59,6 +58,7 @@ GWEN_PLUGIN_DESCRIPTION *GWEN_PluginDescription_new(GWEN_XMLNODE *node){
   const char *p;
 
   GWEN_NEW_OBJECT(GWEN_PLUGIN_DESCRIPTION, pd);
+  pd->refCount=1;
   DBG_MEM_INC("GWEN_PLUGIN_DESCRIPTION", 0);
   GWEN_LIST_INIT(GWEN_PLUGIN_DESCRIPTION, pd);
   p=GWEN_XMLNode_GetProperty(node, "name", 0);
@@ -95,23 +95,37 @@ GWEN_PLUGIN_DESCRIPTION *GWEN_PluginDescription_new(GWEN_XMLNODE *node){
 
 void GWEN_PluginDescription_free(GWEN_PLUGIN_DESCRIPTION *pd){
   if (pd) {
-    DBG_MEM_DEC("GWEN_PLUGIN_DESCRIPTION");
-    GWEN_LIST_FINI(GWEN_PLUGIN_DESCRIPTION, pd);
-    free(pd->path);
-    GWEN_XMLNode_free(pd->xmlNode);
-    free(pd->fileName);
-    free(pd->longDescr);
-    free(pd->shortDescr);
-    free(pd->author);
-    free(pd->version);
-    free(pd->type);
-    free(pd->name);
-    GWEN_FREE_OBJECT(pd);
+    assert(pd->refCount);
+    if (pd->refCount==1) {
+      DBG_MEM_DEC("GWEN_PLUGIN_DESCRIPTION");
+      GWEN_LIST_FINI(GWEN_PLUGIN_DESCRIPTION, pd);
+      free(pd->path);
+      GWEN_XMLNode_free(pd->xmlNode);
+      free(pd->fileName);
+      free(pd->longDescr);
+      free(pd->shortDescr);
+      free(pd->author);
+      free(pd->version);
+      free(pd->type);
+      free(pd->name);
+      pd->refCount=0;
+      GWEN_FREE_OBJECT(pd);
+    }
+    else
+      pd->refCount--;
   }
 }
 
 
 
+void GWEN_PluginDescription_Attach(GWEN_PLUGIN_DESCRIPTION *pd) {
+  assert(pd);
+  assert(pd->refCount);
+  pd->refCount++;
+}
+
+
+
 GWEN_PLUGIN_DESCRIPTION*
 GWEN_PluginDescription_dup(const GWEN_PLUGIN_DESCRIPTION *pd) {
   GWEN_PLUGIN_DESCRIPTION *np;
@@ -119,6 +133,7 @@ GWEN_PluginDescription_dup(const GWEN_PLUGIN_DESCRIPTION *pd) {
 
   assert(pd);
   GWEN_NEW_OBJECT(GWEN_PLUGIN_DESCRIPTION, np);
+  np->refCount=1;
   DBG_MEM_INC("GWEN_PLUGIN_DESCRIPTION", 0);
   GWEN_LIST_INIT(GWEN_PLUGIN_DESCRIPTION, np);
 
