@@ -54,6 +54,9 @@ typedef struct GWEN_STRINGLISTSTRUCT GWEN_STRINGLIST;
 
 
 GWENHYWFAR_API GWEN_STRINGLIST *GWEN_StringList_new();
+
+GWENHYWFAR_API GWEN_STRINGLIST *GWEN_StringList_fromTabString(const char *s, int checkDup);
+
 GWENHYWFAR_API void GWEN_StringList_free(GWEN_STRINGLIST *sl);
 GWENHYWFAR_API
   GWEN_STRINGLIST *GWEN_StringList_dup(const GWEN_STRINGLIST *sl);
@@ -143,6 +146,13 @@ GWENHYWFAR_API int GWEN_StringList_RemoveString(GWEN_STRINGLIST *sl,
 GWENHYWFAR_API int GWEN_StringList_HasString(const GWEN_STRINGLIST *sl,
                                              const char *s);
 
+/**
+ * Returns the position of the given string within the stringlist.
+ *
+ * @return position, -1 if not found
+ */
+GWENHYWFAR_API int GWEN_StringList_GetStringPos(const GWEN_STRINGLIST *sl, const char *s);
+
 
 /** Traverses the list, calling the callback function 'func' on
  * each list element.  Traversal will stop when 'func' returns a
