@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id: crypttoken.h 1113 2007-01-10 09:14:16Z martin $
     begin       : Wed Mar 16 2005
     copyright   : (C) 2005 by Martin Preuss
     email       : martin@libchipcard.de
@@ -14,6 +11,8 @@
 # include <config.h>
 #endif
 
+#define DISABLE_DEBUGLOG
+
 
 #include "cryptkeyrsa_p.h"
 #include <gwenhywfar/misc.h>
@@ -1057,7 +1056,7 @@ GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModExp(unsigned int nbytes,
   if (key==NULL) {
     DBG_INFO(GWEN_LOGDOMAIN,
 	     "Internal error: Bad RSA key group");
-    GWEN_DB_Dump(dbKey, stderr, 2);
+    GWEN_DB_Dump(dbKey, 2);
     GWEN_DB_Group_free(dbKey);
     return NULL;
   }
@@ -1115,7 +1114,7 @@ GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModPrivExp(unsigned int nbytes,
   if (key==NULL) {
     DBG_INFO(GWEN_LOGDOMAIN,
 	     "Internal error: Bad RSA key group");
-    GWEN_DB_Dump(dbKey, stderr, 2);
+    GWEN_DB_Dump(dbKey, 2);
     GWEN_DB_Group_free(dbKey);
     return NULL;
   }
