@@ -32,7 +32,7 @@ extern "C" {
 /**
  * Creates a key pair and returns a pointer to it.
  * @return NULL on error, otherwise a pointer to two keys: public and private
- * @param nbits number of bytes
+ * @param nbytes number of bytes
  * @param use65537e !=0: use the value 65537 for the public exponent, otherwise let the
  *  library choose the exponent by its own
  */
@@ -41,6 +41,20 @@ GWENHYWFAR_API int GWEN_Crypt_KeyRsa_GeneratePair(unsigned int nbytes,
 						  GWEN_CRYPT_KEY **pPubKey,
 						  GWEN_CRYPT_KEY **pSecretKey);
 
+
+/**
+ * Creates a key pair and returns a pointer to it.
+ * This function allows for more precise control over the size of the created key because it
+ * lets you specify the size in bits rather than in bytes.
+ * @return NULL on error, otherwise a pointer to two keys: public and private
+ * @param nbits number of bits
+ * @param use65537e !=0: use the value 65537 for the public exponent, otherwise let the
+ *  library choose the exponent by its own
+ */
+GWENHYWFAR_API int GWEN_Crypt_KeyRsa_GeneratePair2(unsigned int nbits, int use65537e,
+						   GWEN_CRYPT_KEY **pPubKey,
+						   GWEN_CRYPT_KEY **pSecretKey);
+
 GWENHYWFAR_API GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_dup(const GWEN_CRYPT_KEY *k);
 
 GWENHYWFAR_API GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromDb(GWEN_DB_NODE *db);
@@ -56,6 +70,14 @@ GWENHYWFAR_API GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModExp(unsigned int nbytes,
 							    const uint8_t *pExponent,
 							    uint32_t lExponent);
 
+GWENHYWFAR_API GWEN_CRYPT_KEY *GWEN_Crypt_KeyRsa_fromModPrivExp(unsigned int nbytes,
+								const uint8_t *pModulus,
+								uint32_t lModulus,
+								const uint8_t *pExponent,
+								uint32_t lExponent,
+								const uint8_t *pPrivExponent,
+								uint32_t lPrivExponent);
+
 GWENHYWFAR_API uint32_t GWEN_Crypt_KeyRsa_GetFlags(const GWEN_CRYPT_KEY *k);
 GWENHYWFAR_API void GWEN_Crypt_KeyRsa_SetFlags(GWEN_CRYPT_KEY *k,
                                                uint32_t fl);
