@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id: crypttoken.h 1113 2007-01-10 09:14:16Z martin $
     begin       : Wed Mar 16 2005
     copyright   : (C) 2005 by Martin Preuss
     email       : martin@libchipcard.de
@@ -14,6 +11,8 @@
 # include <config.h>
 #endif
 
+#define DISABLE_DEBUGLOG
+
 
 #include "ctplugin_p.h"
 #include "i18n_l.h"
@@ -323,6 +322,8 @@ int GWEN_Crypt_Token_PluginManager_CheckToken(GWEN_PLUGIN_MANAGER *pm,
       progressId=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
 					GWEN_GUI_PROGRESS_ALLOW_EMBED |
 					GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+                                        GWEN_GUI_PROGRESS_SHOW_LOG |
+					GWEN_GUI_PROGRESS_ALWAYS_SHOW_LOG |
 					GWEN_GUI_PROGRESS_SHOW_ABORT,
 					I18N("Determining plugin module..."),
 					NULL,
