@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
- -------------------
- cvs         : $Id: db.c 1231 2007-06-12 01:54:15Z martin $
  begin       : Tue Sep 09 2003
- copyright   : (C) 2003 by Martin Preuss
+ copyright   : (C) 2003-2010 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -126,12 +123,9 @@ int GWEN_DB_ReadFileAs(GWEN_DB_NODE *db,
                        const char *fname,
                        const char *type,
                        GWEN_DB_NODE *params,
-		       uint32_t dbflags,
-		       uint32_t guiid,
-		       int msecs){
-  GWEN_IO_LAYER *io;
+		       uint32_t dbflags){
+  GWEN_SYNCIO *sio;
   GWEN_DBIO *dbio;
-  int fd;
   int rv;
 
   dbio=GWEN_DBIO_GetPlugin(type);
@@ -140,30 +134,21 @@ int GWEN_DB_ReadFileAs(GWEN_DB_NODE *db,
     return GWEN_ERROR_NOT_SUPPORTED;
   }
 
-  fd=open(fname, O_RDONLY);
-  if (fd==-1) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "open(%s, O_RDONLY): %s", fname, strerror(errno));
-    return GWEN_ERROR_IO;
-  }
-
-  /* create io layer for this file (readonly) */
-  io=GWEN_Io_LayerFile_new(fd, -1);
-  assert(io);
-
-  rv=GWEN_Io_Manager_RegisterLayer(io);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
-    GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-    GWEN_Io_Layer_free(io);
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
     return rv;
   }
 
-  rv=GWEN_DBIO_Import(dbio, io, db, params, dbflags, guiid, msecs);
-  if (rv) {
+  rv=GWEN_DBIO_Import(dbio, sio, db, params, dbflags);
+  if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
   }
-  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-  GWEN_Io_Layer_free(io);
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
 
   return rv;
 }
@@ -174,9 +159,7 @@ int GWEN_DB_WriteFileAs(GWEN_DB_NODE *db,
                         const char *fname,
                         const char *type,
                         GWEN_DB_NODE *params,
-			uint32_t dbflags,
-			uint32_t guiid,
-			int msecs){
+			uint32_t dbflags){
   int rv;
   GWEN_DBIO *dbio;
 
@@ -186,7 +169,7 @@ int GWEN_DB_WriteFileAs(GWEN_DB_NODE *db,
     return GWEN_ERROR_NOT_SUPPORTED;
   }
 
-  rv=GWEN_DBIO_ExportToFile(dbio, fname, db, params, dbflags, guiid, msecs);
+  rv=GWEN_DBIO_ExportToFile(dbio, fname, db, params, dbflags);
   if (rv) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
     return rv;
@@ -524,14 +507,12 @@ int GWEN_DB_WriteToFastBuffer(GWEN_DB_NODE *node,
 
 
 int GWEN_DB_WriteToIo(GWEN_DB_NODE *node,
-		      GWEN_IO_LAYER *io,
-		      uint32_t dbflags,
-		      uint32_t guiid,
-		      int msecs) {
+		      GWEN_SYNCIO *sio,
+		      uint32_t dbflags) {
   int rv;
   GWEN_FAST_BUFFER *fb;
 
-  fb=GWEN_FastBuffer_new(512, io, guiid, msecs);
+  fb=GWEN_FastBuffer_new(512, sio);
   if (dbflags & GWEN_DB_FLAGS_DOSMODE)
     GWEN_FastBuffer_AddFlags(fb, GWEN_FAST_BUFFER_FLAGS_DOSMODE);
   rv=GWEN_DB_WriteGroupToIoLayer(node, fb, dbflags, 0);
@@ -546,52 +527,10 @@ int GWEN_DB_WriteToIo(GWEN_DB_NODE *node,
 
 
 
-int GWEN_DB_WriteToFd(GWEN_DB_NODE *n, int fd, uint32_t dbflags, uint32_t guiid, int msecs){
-  GWEN_IO_LAYER *io;
-  int rv;
-
-  /* create io layer for this file */
-  io=GWEN_Io_LayerFile_new(-1, fd);
-  assert(io);
-  GWEN_Io_Layer_AddFlags(io, GWEN_IO_LAYER_FLAGS_DONTCLOSE);
-
-  rv=GWEN_Io_Manager_RegisterLayer(io);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
-    GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  /* write to file */
-  rv=GWEN_DB_WriteToIo(n, io, dbflags, guiid, msecs);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 1000);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  /* close io layer */
-  rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, guiid, 30000);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 1000);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  GWEN_Io_Layer_free(io);
-
-  return 0;
-}
-
-
-
-int GWEN_DB_WriteFile(GWEN_DB_NODE *n, const char *fname, uint32_t dbflags, uint32_t guiid, int msecs){
-  int fd;
+int GWEN_DB_WriteFile(GWEN_DB_NODE *n, const char *fname, uint32_t dbflags){
   int rv;
   GWEN_FSLOCK *lck=0;
+  GWEN_SYNCIO *sio;
 
   /* if locking requested */
   if (dbflags & GWEN_DB_FLAGS_LOCKFILE) {
@@ -610,25 +549,18 @@ int GWEN_DB_WriteFile(GWEN_DB_NODE *n, const char *fname, uint32_t dbflags, uint
   }
 
   /* open file */
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_CreateAlways);
   if (dbflags & GWEN_DB_FLAGS_APPEND_FILE)
-    fd=open(fname, O_RDWR | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
-  else
-    fd=open(fname, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
-  if (fd==-1) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Error opening file \"%s\": %s",
-              fname,
-              strerror(errno));
-    if (lck) {
-      GWEN_FSLock_Unlock(lck);
-      GWEN_FSLock_free(lck);
-    }
-    return GWEN_ERROR_IO;
-  }
-
-  rv=GWEN_DB_WriteToFd(n, fd, dbflags, guiid, msecs);
+    GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_APPEND);
+  GWEN_SyncIo_AddFlags(sio,
+		       GWEN_SYNCIO_FILE_FLAGS_READ |
+		       GWEN_SYNCIO_FILE_FLAGS_WRITE |
+		       GWEN_SYNCIO_FILE_FLAGS_UREAD |
+                       GWEN_SYNCIO_FILE_FLAGS_UWRITE);
+  rv=GWEN_SyncIo_Connect(sio);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    close(fd);
+    GWEN_SyncIo_free(sio);
     if (lck) {
       GWEN_FSLock_Unlock(lck);
       GWEN_FSLock_free(lck);
@@ -636,17 +568,30 @@ int GWEN_DB_WriteFile(GWEN_DB_NODE *n, const char *fname, uint32_t dbflags, uint
     return rv;
   }
 
-  if (close(fd)) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Error closing file \"%s\": %s",
-	      fname,
-	      strerror(errno));
+  rv=GWEN_DB_WriteToIo(n, sio, dbflags);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
     if (lck) {
       GWEN_FSLock_Unlock(lck);
       GWEN_FSLock_free(lck);
     }
-    return GWEN_ERROR_IO;
+    return rv;
   }
 
+  rv=GWEN_SyncIo_Disconnect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    if (lck) {
+      GWEN_FSLock_Unlock(lck);
+      GWEN_FSLock_free(lck);
+    }
+    return rv;
+  }
+  GWEN_SyncIo_free(sio);
+
   /* remove lock, if any */
   if (lck) {
     GWEN_FSLOCK_RESULT res;
@@ -896,7 +841,7 @@ int GWEN_DB_ReadFromFastBuffer(GWEN_DB_NODE *n,
 	    p++;
 	  if (!*p) {
 	    DBG_INFO(GWEN_LOGDOMAIN, "Missing 2nd token");
-            GWEN_Buffer_Dump(lbuf, stderr, 2);
+            GWEN_Buffer_Dump(lbuf, 2);
 	    GWEN_Buffer_free(tbuf);
 	    GWEN_Buffer_free(lbuf);
 	    return GWEN_ERROR_BAD_DATA;
@@ -1021,83 +966,57 @@ int GWEN_DB_ReadFromFastBuffer(GWEN_DB_NODE *n,
 
 
 
-int GWEN_DB_ReadFromIo(GWEN_DB_NODE *n,
-		       GWEN_IO_LAYER *io,
-		       uint32_t dbflags,
-		       uint32_t guiid,
-		       int msecs) {
+int GWEN_DB_ReadFromIo(GWEN_DB_NODE *n, GWEN_SYNCIO *sio, uint32_t dbflags) {
   GWEN_FAST_BUFFER *fb;
   int rv;
 
   /* prepare fast buffer */
-  fb=GWEN_FastBuffer_new(1024, io, guiid, msecs);
+  fb=GWEN_FastBuffer_new(1024, sio);
   if (dbflags & GWEN_DB_FLAGS_DOSMODE)
     GWEN_FastBuffer_AddFlags(fb, GWEN_FAST_BUFFER_FLAGS_DOSMODE);
 
   /* read from it */
   rv=GWEN_DB_ReadFromFastBuffer(n, fb, dbflags);
-  if (rv) {
+  if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
     GWEN_FastBuffer_free(fb);
     return rv;
   }
 
   GWEN_FastBuffer_free(fb);
+
   return 0;
 }
 
 
 
-int GWEN_DB_ReadFromFd(GWEN_DB_NODE *n,
-                       int fd,
-		       uint32_t dbflags,
-		       uint32_t guiid,
-		       int msecs) {
-  GWEN_IO_LAYER *io;
-  int rv;
-
-  /* create io layer for this file (readonly) */
-  io=GWEN_Io_LayerFile_new(fd, -1);
-  assert(io);
-  GWEN_Io_Layer_AddFlags(io, GWEN_IO_LAYER_FLAGS_DONTCLOSE);
-
-  rv=GWEN_Io_Manager_RegisterLayer(io);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  rv=GWEN_DB_ReadFromIo(n, io, dbflags, guiid, msecs);
-  if (rv) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-  }
-  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-  GWEN_Io_Layer_free(io);
-
-  return rv;
-}
-
-
-
 int GWEN_DB_ReadFile(GWEN_DB_NODE *n,
 		     const char *fname,
-		     uint32_t dbflags,
-		     uint32_t guiid,
-		     int msecs) {
-  int fd;
+		     uint32_t dbflags) {
+  GWEN_SYNCIO *sio;
   int rv;
 
-  fd=open(fname, O_RDONLY);
-  if (fd==-1) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "open(%s, O_RDONLY): %s", fname, strerror(errno));
-    return GWEN_ERROR_IO;
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
 
-  rv=GWEN_DB_ReadFromFd(n, fd, dbflags, guiid, msecs);
-  close(fd);
+  /* read from it */
+  rv=GWEN_DB_ReadFromIo(n, sio, dbflags);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
+    return rv;
+  }
 
-  return rv;
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
+  return 0;
 }
 
 
@@ -1105,90 +1024,44 @@ int GWEN_DB_ReadFile(GWEN_DB_NODE *n,
 int GWEN_DB_ReadFromString(GWEN_DB_NODE *n,
 			   const char *str,
                            int len,
-			   uint32_t dbflags,
-			   uint32_t guiid,
-			   int msecs) {
-  GWEN_IO_LAYER *io;
+			   uint32_t dbflags) {
+  GWEN_SYNCIO *sio;
   int rv;
 
   if (len==0)
     len=strlen(str);
 
-  /* create io layer */
-  io=GWEN_Io_LayerMemory_fromString((const uint8_t*)str, len);
-  assert(io);
-
-  rv=GWEN_Io_Manager_RegisterLayer(io);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
-    GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-    GWEN_Io_Layer_free(io);
+  sio=GWEN_SyncIo_Memory_fromBuffer((const uint8_t*) str, len);
+  rv=GWEN_DB_ReadFromIo(n, sio, dbflags);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
     return rv;
   }
 
-  rv=GWEN_DB_ReadFromIo(n, io, dbflags, guiid, msecs);
-  if (rv) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-  }
-  GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-  GWEN_Io_Layer_free(io);
+  GWEN_SyncIo_free(sio);
 
-  return rv;
+  return 0;
 }
 
 
 
 int GWEN_DB_WriteToBuffer(GWEN_DB_NODE *n,
 			  GWEN_BUFFER *buf,
-			  uint32_t dbflags,
-			  uint32_t guiid,
-			  int msecs) {
-  GWEN_IO_LAYER *io;
+			  uint32_t dbflags) {
+  GWEN_SYNCIO *sio;
   int rv;
 
-  /* create io layer */
-  io=GWEN_Io_LayerMemory_new(buf);
-  assert(io);
-
-  rv=GWEN_Io_Manager_RegisterLayer(io);
-  if (rv) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Internal error: Could not register io layer (%d)", rv);
-    GWEN_Io_Layer_DisconnectRecursively(io, NULL, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, msecs);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  rv=GWEN_DB_WriteToIo(n, io, dbflags, guiid, msecs);
+  /* create SyncIO, don't take over buf */
+  sio=GWEN_SyncIo_Memory_new(buf, 0);
+  rv=GWEN_DB_WriteToIo(n, sio, dbflags);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
+    GWEN_SyncIo_free(sio);
     return rv;
   }
 
-  /* flush data */
-  rv=GWEN_Io_Layer_WriteString(io, "",
-			       GWEN_IO_REQUEST_FLAGS_FLUSH,
-			       guiid,
-			       30000);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  /* close io layer */
-  rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, guiid, 30000);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, guiid, 1000);
-    GWEN_Io_Layer_free(io);
-    return rv;
-  }
-
-  GWEN_Io_Layer_free(io);
-
+  GWEN_SyncIo_free(sio);
   return 0;
 }
 
