@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Sun Jun 13 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,6 +12,9 @@
 # include <config.h>
 #endif
 
+#define DISABLE_DEBUGLOG
+
+
 #include "tlv_p.h"
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/inherit.h>
@@ -204,7 +204,7 @@ GWEN_TLV *GWEN_TLV_fromBuffer(GWEN_BUFFER *mbuf, int isBerTlv) {
         j+=(unsigned char)(p[pos]);
       } /* 0x82 */
       else {
-        DBG_ERROR(GWEN_LOGDOMAIN, "Unexpected tag length modifier %02x", j);
+	DBG_ERROR(GWEN_LOGDOMAIN, "Unexpected tag length modifier %02x at %d", j, pos);
         return 0;
       }
     } /* if tag length modifier */
