@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
- -------------------
- cvs         : $Id$
  begin       : Sat Jun 28 2003
- copyright   : (C) 2003 by Martin Preuss
+ copyright   : (C) 2003-2010 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -40,10 +37,8 @@
 #include <gwenhywfar/text.h>
 #include <gwenhywfar/path.h>
 #include <gwenhywfar/fastbuffer.h>
-#include <gwenhywfar/io_file.h>
-#include <gwenhywfar/io_memory.h>
-#include <gwenhywfar/io_buffered.h>
-#include <gwenhywfar/iomanager.h>
+#include <gwenhywfar/syncio_file.h>
+#include <gwenhywfar/syncio_memory.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -91,6 +86,7 @@ void GWEN_XMLProperty_free(GWEN_XMLPROPERTY *p){
   if (p) {
     GWEN_Memory_dealloc(p->name);
     GWEN_Memory_dealloc(p->value);
+    GWEN_Memory_dealloc(p->nameSpace);
     GWEN_FREE_OBJECT(p);
   }
 }
@@ -98,7 +94,13 @@ void GWEN_XMLProperty_free(GWEN_XMLPROPERTY *p){
 
 
 GWEN_XMLPROPERTY *GWEN_XMLProperty_dup(const GWEN_XMLPROPERTY *p){
-  return GWEN_XMLProperty_new(p->name, p->value);
+  GWEN_XMLPROPERTY *pp;
+
+  pp=GWEN_XMLProperty_new(p->name, p->value);
+  if (p->nameSpace)
+    pp->nameSpace=strdup(p->nameSpace);
+
+  return pp;
 }
 
 
@@ -400,7 +402,7 @@ GWEN_XMLNODE *GWEN_XMLNode_Next(const GWEN_XMLNODE *n) {
 }
 
 
-void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, FILE *f, int ind) {
+void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, int ind) {
   GWEN_XMLPROPERTY *p;
   GWEN_XMLNODE *c;
   int i;
@@ -409,59 +411,59 @@ void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, FILE *f, int ind) {
   assert(n);
 
   for(i=0; i<ind; i++)
-    fprintf(f, " ");
+    fprintf(stderr, " ");
 
   simpleTag=0;
   if (n->type==GWEN_XMLNodeTypeTag) {
     if (n->data)
-      fprintf(f, "<%s", n->data);
+      fprintf(stderr, "<%s", n->data);
     else
-      fprintf(f, "<UNKNOWN");
+      fprintf(stderr, "<UNKNOWN");
     p=n->properties;
     while (p) {
       if (p->value)
-        fprintf(f, " %s=\"%s\"", p->name, p->value);
+        fprintf(stderr, " %s=\"%s\"", p->name, p->value);
       else
-        fprintf(f, " %s", p->name);
+        fprintf(stderr, " %s", p->name);
       p=p->next;
     }
 
     if (n->data) {
       if (n->data[0]=='?') {
         simpleTag=1;
-        fprintf(f, "?");
+        fprintf(stderr, "?");
       }
       else if (n->data[0]=='!') {
         simpleTag=1;
       }
     }
 
-    fprintf(f, ">\n");
+    fprintf(stderr, ">\n");
     if (!simpleTag) {
       c=GWEN_XMLNode_GetChild(n);
       while(c) {
-        GWEN_XMLNode_Dump(c, f, ind+2);
+        GWEN_XMLNode_Dump(c, ind+2);
         c=GWEN_XMLNode_Next(c);
       }
       for(i=0; i<ind; i++)
-        fprintf(f, " ");
+        fprintf(stderr, " ");
       if (n->data)
-        fprintf(f, "</%s>\n", n->data);
+        fprintf(stderr, "</%s>\n", n->data);
       else
-        fprintf(f, "</UNKNOWN>\n");
+        fprintf(stderr, "</UNKNOWN>\n");
     }
   }
   else if (n->type==GWEN_XMLNodeTypeData) {
     if (n->data) {
-      fprintf(f, "%s\n", n->data);
+      fprintf(stderr, "%s\n", n->data);
     }
   }
   else if (n->type==GWEN_XMLNodeTypeComment) {
-    fprintf(f, "<!--");
+    fprintf(stderr, "<!--");
     if (n->data) {
-      fprintf(f, "%s", n->data);
+      fprintf(stderr, "%s", n->data);
     }
-    fprintf(f, "-->\n");
+    fprintf(stderr, "-->\n");
   }
   else {
     DBG_ERROR(GWEN_LOGDOMAIN, "Unknown tag type (%d)", n->type);
@@ -1568,6 +1570,7 @@ int GWEN_XMLNode_NormalizeNameSpaces(GWEN_XMLNODE *n) {
 int GWEN_XMLNode_StripNamespaces(GWEN_XMLNODE *n) {
   if (n && n->type==GWEN_XMLNodeTypeTag && n->data) {
     GWEN_XMLNODE *nn;
+    GWEN_XMLPROPERTY *pp;
 
     if (n->nameSpace==0) {
       char *p;
@@ -1587,6 +1590,29 @@ int GWEN_XMLNode_StripNamespaces(GWEN_XMLNODE *n) {
       }
     }
 
+    pp=n->properties;
+    while(pp) {
+      if (pp->nameSpace==0) {
+	char *p;
+
+	p=strchr(pp->name, ':');
+	if (p) {
+	  int len=p-pp->name;
+	  char *s;
+
+	  pp->nameSpace=(char*)GWEN_Memory_malloc(len);
+	  assert(pp->nameSpace);
+	  memmove(pp->nameSpace, pp->name, len);
+	  pp->nameSpace[len-1]=0;
+	  s=GWEN_Memory_strdup(p+1);
+	  free(pp->name);
+	  pp->name=s;
+	}
+      }
+
+      pp=pp->next;
+    }
+
     nn=GWEN_XMLNode_List_First(n->children);
     while(nn) {
       int rv;
@@ -1679,7 +1705,7 @@ void GWEN_XMLNode_Path_Dump(GWEN_XMLNODE_PATH *np){
   }
   for (i=0; i<np->pos; i++) {
     DBG_NOTICE(GWEN_LOGDOMAIN, "Path entry %d:", i);
-    GWEN_XMLNode_Dump(np->nodes[i], stderr, 1);
+    GWEN_XMLNode_Dump(np->nodes[i], 1);
   }
 }
 
