@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
- -------------------
- cvs         : $Id$
  begin       : Sat Jun 28 2003
- copyright   : (C) 2003 by Martin Preuss
+ copyright   : (C) 2003-2010 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -29,11 +26,10 @@
 #define GWENHYWFAR_XML_H
 
 #include <gwenhywfar/gwenhywfarapi.h>
-#include <gwenhywfar/bufferedio.h>
 #include <gwenhywfar/stringlist.h>
 #include <gwenhywfar/types.h>
 #include <gwenhywfar/list2.h>
-#include <gwenhywfar/iolayer.h>
+#include <gwenhywfar/syncio.h>
 
 #include <stdio.h>
 
@@ -591,7 +587,13 @@ int GWEN_XML_ReadFromFastBuffer(GWEN_XML_CONTEXT *ctx, GWEN_FAST_BUFFER *fb);
  * Reads a single element (and all its sub-elements) from an IO layer.
  */
 GWENHYWFAR_API
-int GWEN_XML_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io);
+int GWEN_XMLContext_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_SYNCIO *io);
+
+GWENHYWFAR_API
+int GWEN_XMLContext_ReadFromFile(GWEN_XML_CONTEXT *ctx, const char *fname);
+
+GWENHYWFAR_API
+int GWEN_XMLContext_ReadFromString(GWEN_XML_CONTEXT *ctx, const char *text);
 
 
 /**
@@ -607,13 +609,12 @@ GWEN_XMLNODE *GWEN_XMLNode_fromString(const char *s,
 				      uint32_t flags);
 
 /**
- * Writes a tag and all its subnodes to the given io layer (which is expected to be
- * of type GWEN_IO_LAYER_BUFFERED, see @ref GWEN_Io_LayerBuffered_new).
+ * Writes a tag and all its subnodes to the given io layer.
  */
 GWENHYWFAR_API
 int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
 			       GWEN_XML_CONTEXT *ctx,
-			       GWEN_IO_LAYER *io);
+			       GWEN_SYNCIO *sio);
 
 /**
  * Writes a tag and all its subnodes to the given file.
@@ -731,7 +732,7 @@ void GWEN_XMLNode_SetIntValue(GWEN_XMLNODE *n,
  * Dumps the content of the given XML node and all its children.
  */
 GWENHYWFAR_API
-void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, FILE *f, int ind);
+void GWEN_XMLNode_Dump(const GWEN_XMLNODE *n, int ind);
 /*@}*/
 
 /*@}*/ /* defgroup */
