@@ -1,5 +1,5 @@
 /***************************************************************************
- copyright   : (C) 2007 by Martin Preuss
+ copyright   : (C) 2007-2010 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -185,8 +185,8 @@ int GWEN_XMLNode__WriteToStream(const GWEN_XMLNODE *n,
 
 
 int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
-                               GWEN_XML_CONTEXT *ctx,
-			       GWEN_IO_LAYER *io){
+			       GWEN_XML_CONTEXT *ctx,
+			       GWEN_SYNCIO *sio){
   const GWEN_XMLNODE *nn;
   const GWEN_XMLNODE *nchild;
   const GWEN_XMLNODE *nheader;
@@ -198,7 +198,7 @@ int GWEN_XMLNode_WriteToStream(const GWEN_XMLNODE *n,
   nchild=GWEN_XMLNode_GetChild(n);
   nheader=GWEN_XMLNode_GetHeader(n);
 
-  fb=GWEN_FastBuffer_new(512, io, GWEN_XmlCtx_GetGuiId(ctx), GWEN_XmlCtx_GetTimeout(ctx));
+  fb=GWEN_FastBuffer_new(512, sio);
 
   if (nheader && (flags & GWEN_XML_FLAGS_HANDLE_HEADERS)) {
     uint32_t lflags;
@@ -279,43 +279,34 @@ int GWEN_XMLNode_WriteFile(const GWEN_XMLNODE *n,
                            const char *fname,
                            uint32_t flags){
   GWEN_XML_CONTEXT *ctx;
-  GWEN_IO_LAYER *io;
-  int fd;
+  GWEN_SYNCIO *sio;
   int rv;
 
-  /* open file for writing */
-  fd=open(fname, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
-  if (fd==-1) {
-    DBG_ERROR(0, "open(%s): %s", fname, strerror(errno));
-    return GWEN_ERROR_IO;
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_CreateAlways);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    return rv;
   }
 
   /* create context and io layers */
-  ctx=GWEN_XmlCtxStore_new(NULL, flags, 0, 10000);
-  io=GWEN_Io_LayerFile_new(-1, fd);
-  GWEN_Io_Manager_RegisterLayer(io);
+  ctx=GWEN_XmlCtxStore_new(NULL, flags);
 
   /* write data to stream */
-  rv=GWEN_XMLNode_WriteToStream(n, ctx, io);
+  rv=GWEN_XMLNode_WriteToStream(n, ctx, sio);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
     GWEN_XmlCtx_free(ctx);
     return rv;
   }
 
   /* close file */
-  rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, GWEN_XmlCtx_GetGuiId(ctx), 30000);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, GWEN_XmlCtx_GetGuiId(ctx), 1000);
-    GWEN_Io_Layer_free(io);
-    GWEN_XmlCtx_free(ctx);
-    return rv;
-  }
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
 
-  GWEN_Io_Layer_free(io);
   GWEN_XmlCtx_free(ctx);
 
   return 0;
@@ -325,35 +316,25 @@ int GWEN_XMLNode_WriteFile(const GWEN_XMLNODE *n,
 
 int GWEN_XMLNode_toBuffer(const GWEN_XMLNODE *n, GWEN_BUFFER *buf, uint32_t flags){
   GWEN_XML_CONTEXT *ctx;
-  GWEN_IO_LAYER *io;
+  GWEN_SYNCIO *sio;
   int rv;
 
+  sio=GWEN_SyncIo_Memory_new(buf, 0);
+
   /* create context and io layers */
-  ctx=GWEN_XmlCtxStore_new(NULL, flags, 0, 10000);
-  io=GWEN_Io_LayerMemory_new(buf);
-  GWEN_Io_Manager_RegisterLayer(io);
+  ctx=GWEN_XmlCtxStore_new(NULL, flags);
 
   /* write data to stream */
-  rv=GWEN_XMLNode_WriteToStream(n, ctx, io);
+  rv=GWEN_XMLNode_WriteToStream(n, ctx, sio);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
+    GWEN_SyncIo_free(sio);
     GWEN_XmlCtx_free(ctx);
     return rv;
   }
 
-  /* close file */
-  rv=GWEN_Io_Layer_DisconnectRecursively(io, NULL, 0, 0, 30000);
-  if (rv<0) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
-    GWEN_XmlCtx_free(ctx);
-    return rv;
-  }
+  GWEN_SyncIo_free(sio);
 
-  GWEN_Io_Layer_free(io);
   GWEN_XmlCtx_free(ctx);
 
   return 0;
@@ -395,11 +376,15 @@ int GWEN_XML__ReadData(GWEN_XML_CONTEXT *ctx,
 
   if (GWEN_Buffer_GetUsedBytes(dbuf)) {
     int rv;
+    const char *s;
 
-    rv=GWEN_XmlCtx_AddData(ctx, GWEN_Buffer_GetStart(dbuf));
-    if (rv) {
-      GWEN_Buffer_free(dbuf);
-      return rv;
+    s=GWEN_Buffer_GetStart(dbuf);
+    if (*s) {
+      rv=GWEN_XmlCtx_AddData(ctx, s);
+      if (rv) {
+	GWEN_Buffer_free(dbuf);
+	return rv;
+      }
     }
   }
   GWEN_Buffer_free(dbuf);
@@ -411,19 +396,16 @@ int GWEN_XML__ReadData(GWEN_XML_CONTEXT *ctx,
 
 int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 		      GWEN_FAST_BUFFER *fb,
-		      GWEN_UNUSED uint32_t flags){
+                      GWEN_UNUSED uint32_t flags,
+                      GWEN_BUFFER *dbuf){
   int chr;
   unsigned char uc=0;
-  GWEN_BUFFER *dbuf;
   int rv;
 
-  dbuf=GWEN_Buffer_new(0, 256, 0, 1);
-
   /* skip blanks */
   for (;;) {
     GWEN_FASTBUFFER_READBYTE(fb, chr);
     if (chr<0) {
-      GWEN_Buffer_free(dbuf);
       return chr;
     }
     uc=(unsigned char) chr;
@@ -437,7 +419,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
     for (;;) {
       GWEN_FASTBUFFER_READBYTE(fb, chr);
       if (chr<0) {
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
       uc=(unsigned char) chr;
@@ -449,7 +430,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 
     rv=GWEN_XmlCtx_StartTag(ctx, GWEN_Buffer_GetStart(dbuf));
     if (rv) {
-      GWEN_Buffer_free(dbuf);
       return rv;
     }
     if (uc!='>') {
@@ -457,7 +437,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 	/* skip blanks, expect '>' */
 	GWEN_FASTBUFFER_READBYTE(fb, chr);
 	if (chr<0) {
-	  GWEN_Buffer_free(dbuf);
 	  return chr;
 	}
 	uc=(unsigned char) chr;
@@ -467,24 +446,20 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
     }
     if (uc!='>') {
       DBG_ERROR(GWEN_LOGDOMAIN, "Unexpected character");
-      GWEN_Buffer_free(dbuf);
       return GWEN_ERROR_BAD_DATA;
     }
 
     /* tag finished */
     rv=GWEN_XmlCtx_EndTag(ctx, 0);
     if (rv) {
-      GWEN_Buffer_free(dbuf);
       return rv;
     }
-    GWEN_Buffer_free(dbuf);
     return 0;
   }
   else if (uc=='!') {
     /* check for comment */
     GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
     if (chr<0) {
-      GWEN_Buffer_free(dbuf);
       return chr;
     }
     uc=(unsigned char) chr;
@@ -492,7 +467,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       fb->bufferReadPos++;
       GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
       if (chr<0) {
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
       uc=(unsigned char) chr;
@@ -506,7 +480,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 	  GWEN_FASTBUFFER_READBYTE(fb, chr);
 	  if (chr<0) {
 	    GWEN_Buffer_free(cbuf);
-	    GWEN_Buffer_free(dbuf);
 	    return chr;
 	  }
 	  uc=(unsigned char) chr;
@@ -521,11 +494,9 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 	      rv=GWEN_XmlCtx_AddComment(ctx, GWEN_Buffer_GetStart(cbuf));
 	      if (rv) {
 		GWEN_Buffer_free(cbuf);
-		GWEN_Buffer_free(dbuf);
                 return rv;
 	      }
 	      GWEN_Buffer_free(cbuf);
-	      GWEN_Buffer_free(dbuf);
 	      return 0;
 	    }
 	  }
@@ -550,7 +521,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       if ((fc=='!' && uc=='!') || (fc=='?' && uc=='?')) {
 	GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
 	if (chr<0) {
-	  GWEN_Buffer_free(dbuf);
 	  return chr;
 	}
 	uc=(unsigned char) chr;
@@ -566,11 +536,9 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
     GWEN_FASTBUFFER_READBYTE(fb, chr);
     if (chr<0) {
       if (chr==GWEN_ERROR_EOF) {
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
       else {
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
     }
@@ -581,20 +549,17 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
   /* tag started */
   if (GWEN_Buffer_GetUsedBytes(dbuf)==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Element name missing");
-    GWEN_Buffer_free(dbuf);
     return GWEN_ERROR_BAD_DATA;
   }
 
   rv=GWEN_XmlCtx_StartTag(ctx, GWEN_Buffer_GetStart(dbuf));
   if (rv) {
-    GWEN_Buffer_free(dbuf);
     return rv;
   }
 
   if (uc=='/' || uc=='?' || uc=='!') {
     GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
     if (chr<0) {
-      GWEN_Buffer_free(dbuf);
       return chr;
     }
     uc=(unsigned char) chr;
@@ -602,10 +567,8 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       fb->bufferReadPos++;
       rv=GWEN_XmlCtx_EndTag(ctx, 1);
       if (rv) {
-	GWEN_Buffer_free(dbuf);
 	return rv;
       }
-      GWEN_Buffer_free(dbuf);
       /* tag finished */
       return 0;
     }
@@ -614,10 +577,8 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
   if (uc=='>') {
     rv=GWEN_XmlCtx_EndTag(ctx, 0);
     if (rv) {
-      GWEN_Buffer_free(dbuf);
       return rv;
     }
-    GWEN_Buffer_free(dbuf);
     /* tag finished */
     return 0;
   }
@@ -634,7 +595,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       GWEN_FASTBUFFER_READBYTE(fb, chr);
       if (chr<0) {
 	GWEN_Buffer_free(nbuf);
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
       uc=(unsigned char) chr;
@@ -651,7 +611,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       GWEN_FASTBUFFER_READBYTE(fb, chr);
       if (chr<0) {
 	GWEN_Buffer_free(nbuf);
-	GWEN_Buffer_free(dbuf);
 	return chr;
       }
       uc=(unsigned char) chr;
@@ -667,7 +626,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 	  GWEN_FASTBUFFER_READBYTE(fb, chr);
 	  if (chr<0) {
 	    GWEN_Buffer_free(nbuf);
-	    GWEN_Buffer_free(dbuf);
 	    return chr;
 	  }
 	  uc=(unsigned char) chr;
@@ -688,7 +646,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 			  "Nested element definitions");
 		GWEN_Buffer_free(vbuf);
 		GWEN_Buffer_free(nbuf);
-		GWEN_Buffer_free(dbuf);
 		return GWEN_ERROR_BAD_DATA;
 	      }
 	      else if (GWEN_Buffer_GetUsedBytes(dbuf)) {
@@ -699,7 +656,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 		  if (chr<0) {
 		    GWEN_Buffer_free(vbuf);
 		    GWEN_Buffer_free(nbuf);
-		    GWEN_Buffer_free(dbuf);
 		    return chr;
 		  }
 		  tc=(unsigned char) chr;
@@ -716,7 +672,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 	  DBG_ERROR(GWEN_LOGDOMAIN, "No matching number of quote chars");
 	  GWEN_Buffer_free(vbuf);
 	  GWEN_Buffer_free(nbuf);
-	  GWEN_Buffer_free(dbuf);
 	  return GWEN_ERROR_BAD_DATA;
 	}
 
@@ -731,7 +686,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       if (rv) {
 	GWEN_Buffer_free(vbuf);
 	GWEN_Buffer_free(nbuf);
-	GWEN_Buffer_free(dbuf);
 	return rv;
       }
     }
@@ -748,7 +702,6 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 
     GWEN_FASTBUFFER_PEEKBYTE(fb, chr);
     if (chr<0) {
-      GWEN_Buffer_free(dbuf);
       return chr;
     }
     uc=(unsigned char) chr;
@@ -757,10 +710,8 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
       fb->bufferReadPos++;
       rv=GWEN_XmlCtx_EndTag(ctx, 1);
       if (rv) {
-	GWEN_Buffer_free(dbuf);
 	return rv;
       }
-      GWEN_Buffer_free(dbuf);
       /* tag finished */
       return 0;
     }
@@ -774,17 +725,14 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
   else if (uc=='>') {
     rv=GWEN_XmlCtx_EndTag(ctx, 0);
     if (rv) {
-      GWEN_Buffer_free(dbuf);
       return rv;
     }
-    GWEN_Buffer_free(dbuf);
     /* tag finished */
     return 0;
   }
 
   DBG_ERROR(GWEN_LOGDOMAIN,
 	    "Internal error: Should never reach this point");
-  GWEN_Buffer_free(dbuf);
   return GWEN_ERROR_INTERNAL;
 }
 
@@ -794,9 +742,11 @@ int GWEN_XML__ReadTag(GWEN_XML_CONTEXT *ctx,
 int GWEN_XML_ReadFromFastBuffer(GWEN_XML_CONTEXT *ctx, GWEN_FAST_BUFFER *fb){
   int oks=0;
   int startingDepth;
+  GWEN_BUFFER *workBuf;
 
   startingDepth=GWEN_XmlCtx_GetDepth(ctx);
 
+  workBuf=GWEN_Buffer_new(0, 256, 0, 1);
   GWEN_XmlCtx_ResetFinishedElement(ctx);
   for (;;) {
     int rv;
@@ -804,15 +754,19 @@ int GWEN_XML_ReadFromFastBuffer(GWEN_XML_CONTEXT *ctx, GWEN_FAST_BUFFER *fb){
     GWEN_FASTBUFFER_PEEKBYTE(fb, rv);
     if (rv<0) {
       if (rv!=GWEN_ERROR_EOF || !oks) {
-	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+        DBG_DEBUG(GWEN_LOGDOMAIN, "here (%d), after reading %d bytes",
+                  rv, (int) GWEN_FastBuffer_GetBytesRead(fb));
+        GWEN_Buffer_free(workBuf);
 	return rv;
       }
+      GWEN_Buffer_free(workBuf);
       return 0;
     }
 
     rv=GWEN_XML__ReadData(ctx, fb, GWEN_XmlCtx_GetFlags(ctx));
     if (rv) {
       DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      GWEN_Buffer_free(workBuf);
       return rv;
     }
     oks=1;
@@ -822,17 +776,20 @@ int GWEN_XML_ReadFromFastBuffer(GWEN_XML_CONTEXT *ctx, GWEN_FAST_BUFFER *fb){
       if (rv!=GWEN_ERROR_EOF || !oks ||
 	  (GWEN_XmlCtx_GetDepth(ctx)!=startingDepth)) {
 	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+        GWEN_Buffer_free(workBuf);
 	return rv;
       }
       return 0;
     }
     else if (rv=='<') {
       fb->bufferReadPos++;
-      rv=GWEN_XML__ReadTag(ctx, fb, GWEN_XmlCtx_GetFlags(ctx));
+      rv=GWEN_XML__ReadTag(ctx, fb, GWEN_XmlCtx_GetFlags(ctx), workBuf);
       if (rv) {
 	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+        GWEN_Buffer_free(workBuf);
 	return rv;
       }
+      GWEN_Buffer_Reset(workBuf);
       oks=1;
     }
 
@@ -846,20 +803,18 @@ int GWEN_XML_ReadFromFastBuffer(GWEN_XML_CONTEXT *ctx, GWEN_FAST_BUFFER *fb){
 	      "Not on same level where we started...(%d!=%d)",
 	      GWEN_XmlCtx_GetDepth(ctx), startingDepth);
   }
+  GWEN_Buffer_free(workBuf);
 
   return 0;
 }
 
 
 
-int GWEN_XML__ReadAllFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io){
+int GWEN_XML__ReadAllFromIo(GWEN_XML_CONTEXT *ctx, GWEN_SYNCIO *sio){
   GWEN_FAST_BUFFER *fb;
   int oks=0;
 
-  fb=GWEN_FastBuffer_new(GWEN_XML_BUFFERSIZE,
-			 io,
-			 GWEN_XmlCtx_GetGuiId(ctx),
-			 GWEN_XmlCtx_GetTimeout(ctx));
+  fb=GWEN_FastBuffer_new(GWEN_XML_BUFFERSIZE, sio);
   assert(fb);
   for (;;) {
     int rv;
@@ -883,14 +838,12 @@ int GWEN_XML__ReadAllFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io){
 
 
 
-int GWEN_XML_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io){
+int GWEN_XMLContext_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_SYNCIO *sio){
+#if 0
   GWEN_FAST_BUFFER *fb;
   int rv;
 
-  fb=GWEN_FastBuffer_new(GWEN_XML_BUFFERSIZE,
-			 io,
-			 GWEN_XmlCtx_GetGuiId(ctx),
-			 GWEN_XmlCtx_GetTimeout(ctx));
+  fb=GWEN_FastBuffer_new(GWEN_XML_BUFFERSIZE, sio);
   assert(fb);
   rv=GWEN_XML_ReadFromFastBuffer(ctx, fb);
   if (rv) {
@@ -901,6 +854,76 @@ int GWEN_XML_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io){
 
   GWEN_FastBuffer_free(fb);
   return 0;
+#else
+  int rv;
+
+  rv=GWEN_XML__ReadAllFromIo(ctx, sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    return rv;
+  }
+
+  return rv;
+#endif
+}
+
+
+
+int GWEN_XMLContext_ReadFromFile(GWEN_XML_CONTEXT *ctx, const char *fname) {
+  GWEN_SYNCIO *sio;
+  int rv;
+
+  sio=GWEN_SyncIo_File_new(fname, GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    return rv;
+  }
+
+  rv=GWEN_XML__ReadAllFromIo(ctx, sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
+    return rv;
+  }
+
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
+
+  return 0;
+}
+
+
+
+int GWEN_XMLContext_ReadFromString(GWEN_XML_CONTEXT *ctx, const char *text) {
+  if (text && *text) {
+    GWEN_SYNCIO *sio;
+    int rv;
+    GWEN_BUFFER *tbuf;
+    int i;
+
+    i=strlen(text)+1;
+    tbuf=GWEN_Buffer_new((char*)text, i, i, 0);
+    /* static buffer, don't resize */
+    GWEN_Buffer_SubMode(tbuf, GWEN_BUFFER_MODE_DYNAMIC);
+    GWEN_Buffer_AddMode(tbuf, GWEN_BUFFER_MODE_READONLY);
+    sio=GWEN_SyncIo_Memory_new(tbuf, 0);
+
+    rv=GWEN_XML__ReadAllFromIo(ctx, sio);
+    if (rv<0) {
+      DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+      GWEN_SyncIo_free(sio);
+      GWEN_Buffer_free(tbuf);
+      return rv;
+    }
+
+    GWEN_SyncIo_free(sio);
+    GWEN_Buffer_free(tbuf);
+  }
+  return 0;
 }
 
 
@@ -908,32 +931,31 @@ int GWEN_XML_ReadFromIo(GWEN_XML_CONTEXT *ctx, GWEN_IO_LAYER *io){
 
 int GWEN_XML_ReadFile(GWEN_XMLNODE *n, const char *filepath, uint32_t flags) {
   GWEN_XML_CONTEXT *ctx;
-  GWEN_IO_LAYER *io;
-  int fd;
+  GWEN_SYNCIO *sio;
   int rv;
 
-  fd=open(filepath, O_RDONLY);
-  if (fd==-1) {
-    DBG_ERROR(0, "open(%s): %s", filepath, strerror(errno));
-    return GWEN_ERROR_IO;
-  }
-
-  ctx=GWEN_XmlCtxStore_new(n, flags, 0, 10000);
-  io=GWEN_Io_LayerFile_new(fd, -1);
-  GWEN_Io_Manager_RegisterLayer(io);
-
-  rv=GWEN_XML__ReadAllFromIo(ctx, io);
+  sio=GWEN_SyncIo_File_new(filepath, GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
+    GWEN_SyncIo_free(sio);
+    return rv;
+  }
+
+  ctx=GWEN_XmlCtxStore_new(n, flags);
+  rv=GWEN_XML__ReadAllFromIo(ctx, sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
     GWEN_XmlCtx_free(ctx);
     return rv;
   }
 
-  GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
 
-  GWEN_Io_Layer_free(io);
   GWEN_XmlCtx_free(ctx);
 
   return 0;
@@ -942,32 +964,63 @@ int GWEN_XML_ReadFile(GWEN_XMLNODE *n, const char *filepath, uint32_t flags) {
 
 
 GWEN_XMLNODE *GWEN_XMLNode_fromString(const char *s, int len, uint32_t flags) {
+#if 0
   GWEN_XML_CONTEXT *ctx;
-  GWEN_IO_LAYER *io;
+  GWEN_SYNCIO *sio;
   GWEN_XMLNODE *n;
   int rv;
 
-  n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "doc");
-  ctx=GWEN_XmlCtxStore_new(n, flags, 0, 10000);
-  io=GWEN_Io_LayerMemory_fromString((const uint8_t*)s, len);
-  GWEN_Io_Manager_RegisterLayer(io);
+  if (len==0)
+    len=strlen(s);
+  sio=GWEN_SyncIo_Memory_fromBuffer((const uint8_t*)s, len);
 
-  rv=GWEN_XML__ReadAllFromIo(ctx, io);
+  n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "doc");
+  ctx=GWEN_XmlCtxStore_new(n, flags);
+  rv=GWEN_XML__ReadAllFromIo(ctx, sio);
   if (rv<0) {
     DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
-    GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
-    GWEN_Io_Layer_free(io);
+    GWEN_SyncIo_free(sio);
     GWEN_XmlCtx_free(ctx);
     GWEN_XMLNode_free(n);
     return NULL;
   }
 
-  GWEN_Io_Layer_Disconnect(io, GWEN_IO_REQUEST_FLAGS_FORCE, 0, 1000);
+  GWEN_SyncIo_free(sio);
 
-  GWEN_Io_Layer_free(io);
   GWEN_XmlCtx_free(ctx);
 
   return n;
+#else
+  GWEN_XML_CONTEXT *ctx;
+  GWEN_SYNCIO *sio;
+  GWEN_XMLNODE *n;
+  int rv;
+  GWEN_BUFFER *tbuf;
+
+  tbuf=GWEN_Buffer_new((char*)s, len, len, 0);
+  /* static buffer, don't resize */
+  GWEN_Buffer_SubMode(tbuf, GWEN_BUFFER_MODE_DYNAMIC);
+  GWEN_Buffer_AddMode(tbuf, GWEN_BUFFER_MODE_READONLY);
+  sio=GWEN_SyncIo_Memory_new(tbuf, 0);
+
+  n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "doc");
+  ctx=GWEN_XmlCtxStore_new(n, flags);
+  rv=GWEN_XML__ReadAllFromIo(ctx, sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_XmlCtx_free(ctx);
+    GWEN_XMLNode_free(n);
+    GWEN_SyncIo_free(sio);
+    GWEN_Buffer_free(tbuf);
+    return NULL;
+  }
+
+  GWEN_XmlCtx_free(ctx);
+  GWEN_SyncIo_free(sio);
+  GWEN_Buffer_free(tbuf);
+
+  return n;
+#endif
 }
 
 
