@@ -40,6 +40,7 @@ TYPEMAKER2_TYPE *Typemaker2_Type_new() {
   ty->defines=Typemaker2_Define_List_new();
   ty->codeDefs=Typemaker2_Code_List_new();
   ty->inlines=Typemaker2_Inline_List_new();
+  ty->virtualFns=Typemaker2_VirtualFn_List_new();
 
   ty->structIncludes=GWEN_StringList_new();
   ty->privateIncludes=GWEN_StringList_new();
@@ -65,17 +66,6 @@ void Typemaker2_Type_free(TYPEMAKER2_TYPE *ty) {
       free(ty->inherits);
       free(ty->identifier);
       free(ty->prefix);
-      free(ty->code_construct);
-      free(ty->code_destruct);
-      free(ty->code_assign);
-      free(ty->code_dup);
-      free(ty->code_compare);
-      free(ty->code_fromDb);
-      free(ty->code_toDb);
-      free(ty->code_fromXml);
-      free(ty->code_toXml);
-      free(ty->code_toObject);
-      free(ty->code_fromObject);
 
       free(ty->defaultValue);
       free(ty->presetValue);
@@ -88,6 +78,7 @@ void Typemaker2_Type_free(TYPEMAKER2_TYPE *ty) {
       Typemaker2_Define_List_free(ty->defines);
       Typemaker2_Code_List_free(ty->codeDefs);
       Typemaker2_Inline_List_free(ty->inlines);
+      Typemaker2_VirtualFn_List_free(ty->virtualFns);
 
       GWEN_StringList_free(ty->structIncludes);
       GWEN_StringList_free(ty->privateIncludes);
@@ -440,248 +431,6 @@ void Typemaker2_Type_SetPrefix(TYPEMAKER2_TYPE *ty, const char *s) {
 
 
 
-const char *Typemaker2_Type_GetCodeConstruct(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_construct==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeConstruct(ty->extendsPtr);
-
-  return ty->code_construct;
-}
-
-
-
-void Typemaker2_Type_SetCodeConstruct(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_construct);
-  if (s && *s) ty->code_construct=strdup(s);
-  else ty->code_construct=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeDestruct(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_destruct==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeDestruct(ty->extendsPtr);
-
-  return ty->code_destruct;
-}
-
-
-
-void Typemaker2_Type_SetCodeDestruct(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_destruct);
-  if (s && *s) ty->code_destruct=strdup(s);
-  else ty->code_destruct=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeAssign(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_assign==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeAssign(ty->extendsPtr);
-
-  return ty->code_assign;
-}
-
-
-
-void Typemaker2_Type_SetCodeAssign(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_assign);
-  if (s && *s) ty->code_assign=strdup(s);
-  else ty->code_assign=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeDup(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_dup==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeDup(ty->extendsPtr);
-
-  return ty->code_dup;
-}
-
-
-
-void Typemaker2_Type_SetCodeDup(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_dup);
-  if (s && *s) ty->code_dup=strdup(s);
-  else ty->code_dup=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeCompare(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_compare==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeCompare(ty->extendsPtr);
-
-  return ty->code_compare;
-}
-
-
-
-void Typemaker2_Type_SetCodeCompare(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_compare);
-  if (s && *s) ty->code_compare=strdup(s);
-  else ty->code_compare=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeFromDb(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_fromDb==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeFromDb(ty->extendsPtr);
-
-  return ty->code_fromDb;
-}
-
-
-
-void Typemaker2_Type_SetCodeFromDb(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_fromDb);
-  if (s && *s) ty->code_fromDb=strdup(s);
-  else ty->code_fromDb=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeToDb(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_toDb==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeToDb(ty->extendsPtr);
-
-  return ty->code_toDb;
-}
-
-
-
-void Typemaker2_Type_SetCodeToDb(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_toDb);
-  if (s && *s) ty->code_toDb=strdup(s);
-  else ty->code_toDb=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeFromXml(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_fromXml==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeFromXml(ty->extendsPtr);
-
-  return ty->code_fromXml;
-}
-
-
-
-void Typemaker2_Type_SetCodeFromXml(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_fromXml);
-  if (s && *s) ty->code_fromXml=strdup(s);
-  else ty->code_fromXml=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeToXml(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_toXml==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeToXml(ty->extendsPtr);
-
-  return ty->code_toXml;
-}
-
-
-
-void Typemaker2_Type_SetCodeToXml(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_toXml);
-  if (s && *s) ty->code_toXml=strdup(s);
-  else ty->code_toXml=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeFromObject(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_fromObject==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeFromObject(ty->extendsPtr);
-
-  return ty->code_fromObject;
-}
-
-
-
-void Typemaker2_Type_SetCodeFromObject(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_fromObject);
-  if (s && *s) ty->code_fromObject=strdup(s);
-  else ty->code_fromObject=NULL;
-}
-
-
-
-const char *Typemaker2_Type_GetCodeToObject(const TYPEMAKER2_TYPE *ty) {
-  assert(ty);
-  assert(ty->refCount);
-
-  if (ty->code_toObject==NULL && ty->extendsPtr)
-    return Typemaker2_Type_GetCodeToObject(ty->extendsPtr);
-
-  return ty->code_toObject;
-}
-
-
-
-void Typemaker2_Type_SetCodeToObject(TYPEMAKER2_TYPE *ty, const char *s) {
-  assert(ty);
-  assert(ty->refCount);
-  free(ty->code_toObject);
-  if (s && *s) ty->code_toObject=strdup(s);
-  else ty->code_toObject=NULL;
-}
-
-
-
 TYPEMAKER2_TYPE *Typemaker2_Type_GetExtendsPtr(const TYPEMAKER2_TYPE *ty) {
   assert(ty);
   assert(ty->refCount);
@@ -778,6 +527,14 @@ TYPEMAKER2_INLINE_LIST *Typemaker2_Type_GetInlines(const TYPEMAKER2_TYPE *ty) {
 
 
 
+TYPEMAKER2_VIRTUALFN_LIST *Typemaker2_Type_GetVirtualFns(const TYPEMAKER2_TYPE *ty) {
+  assert(ty);
+  assert(ty->refCount);
+  return ty->virtualFns;
+}
+
+
+
 int Typemaker2_Type_GetNonVolatileMemberCount(const TYPEMAKER2_TYPE *ty) {
   assert(ty);
   assert(ty->refCount);
@@ -935,44 +692,6 @@ int Typemaker2_Type_readXml(TYPEMAKER2_TYPE *ty, GWEN_XMLNODE *node, const char
     }
   }
 
-  /* read code */
-  n=GWEN_XMLNode_FindFirstTag(langNode, "code", NULL, NULL);
-  if (n) {
-    s=GWEN_XMLNode_GetCharValue(n, "construct", NULL);
-    Typemaker2_Type_SetCodeConstruct(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "destruct", NULL);
-    Typemaker2_Type_SetCodeDestruct(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "assign", NULL);
-    Typemaker2_Type_SetCodeAssign(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "dup", NULL);
-    Typemaker2_Type_SetCodeDup(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "compare", NULL);
-    Typemaker2_Type_SetCodeCompare(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "todb", NULL);
-    Typemaker2_Type_SetCodeToDb(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "fromdb", NULL);
-    Typemaker2_Type_SetCodeFromDb(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "toxml", NULL);
-    Typemaker2_Type_SetCodeToXml(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "fromxml", NULL);
-    Typemaker2_Type_SetCodeFromXml(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "toobject", NULL);
-    Typemaker2_Type_SetCodeToObject(ty, s);
-
-    s=GWEN_XMLNode_GetCharValue(n, "fromobject", NULL);
-    Typemaker2_Type_SetCodeFromObject(ty, s);
-  }
-
-
   /* read inlines */
   n=GWEN_XMLNode_FindFirstTag(langNode, "inlines", NULL, NULL);
   if (n) {
@@ -1042,6 +761,28 @@ int Typemaker2_Type_readXml(TYPEMAKER2_TYPE *ty, GWEN_XMLNODE *node, const char
     }
   }
 
+  /* read virtual functions */
+  n=GWEN_XMLNode_FindFirstTag(node, "virtualFns", NULL, NULL);
+  if (n) {
+    GWEN_XMLNODE *nn;
+
+    nn=GWEN_XMLNode_FindFirstTag(n, "fn", NULL, NULL);
+    while(nn) {
+      TYPEMAKER2_VIRTUALFN *vf;
+      int rv;
+
+      vf=Typemaker2_VirtualFn_new();
+      rv=Typemaker2_VirtualFn_readXml(vf, nn);
+      if (rv<0) {
+	DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+	Typemaker2_VirtualFn_free(vf);
+        return rv;
+      }
+      Typemaker2_VirtualFn_List_Add(vf, ty->virtualFns);
+      nn=GWEN_XMLNode_FindNextTag(nn, "fn", NULL, NULL);
+    }
+  }
+
   /* read defaults */
   n=GWEN_XMLNode_FindFirstTag(langNode, "defaults", NULL, NULL);
   if (n) {
