@@ -35,29 +35,18 @@ struct TYPEMAKER2_TYPE {
   char *identifier;
   char *prefix;
 
-  char *code_construct;
-  char *code_destruct;
-  char *code_assign;
-  char *code_dup;
-  char *code_compare;
-  char *code_fromDb;
-  char *code_toDb;
-  char *code_fromXml;
-  char *code_toXml;
-  char *code_toObject;
-  char *code_fromObject;
-
   char *defaultValue;
   char *presetValue;
 
   char *aedb_type;
-  char *baseFileName;;
+  char *baseFileName;
 
   TYPEMAKER2_HEADER_LIST *headers;
   TYPEMAKER2_ENUM_LIST *enums;
   TYPEMAKER2_DEFINE_LIST *defines;
   TYPEMAKER2_CODE_LIST *codeDefs;
   TYPEMAKER2_INLINE_LIST *inlines;
+  TYPEMAKER2_VIRTUALFN_LIST *virtualFns;
 
   GWEN_STRINGLIST *structIncludes;
   GWEN_STRINGLIST *privateIncludes;
